/*
 *  gnu/regexp/REToken.java
 *  Copyright (C) 1998 Wes Biggs
 *
 *  This library is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Library General Public License as published
 *  by the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package gnu.regexp;

abstract class REToken {
  static final String newline; // used by RETokenStart and RETokenEnd

  static {
    newline = System.getProperty("line.separator");
  }

  private REToken m_next = null;
  private int level = 0;
  
  abstract int match(CharIndexed input, int index, int eflags, REMatch mymatch);
  
  protected int next(CharIndexed input, int index, int eflags, REMatch mymatch) {
    return (m_next == null) ? index : m_next.match(input,index,eflags,mymatch);
  }
  
  boolean chain(REToken next) {
    m_next = next;
    return true; // Token was accepted
  }
  
  String toList() {
    String s = new String();
    REToken p = null;
    for (REToken r=this; r != null && r.level >= level; r=r.m_next) {
      if (p != null && p.grouped() && !r.grouped()) s += indentLine();
      s += r.toString();
      p = r;
    }
    return s;
  }
  
  int level() {
    return level;
  }
  
  void setLevel(int newLevel) {
    level = newLevel;
  }
  
  String indentLine() {
    String s = newline;
    for (int i=0; i<level(); i++) s += "  ";
    return s;
  }
  
  boolean grouped() {
    return false;
  }
}
