/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


package sdsu.algorithms.data;

/**
 * This class modifies sequence of numbers. It modifies the sequence so that
 * all numbers in the sequence are between a lower and upper bound. The lower bound
 * may be achieved, but the upper bound is not. That is all numbers will be in the range
 * [lowerBound, upperBound). 
 *
 * @see		sdsu.data.NumberGenerator
 * @version 1.0 18 June 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */

public class  BoundedRangeGenarator implements NumberGenerator
	{
	double range;
	double lowerBound;
	NumberGenerator nestedGenerator;
	
	/**
	 * Creates a generator that only produces numbers between 
	 * lowerBond and upperBound.
	 * @exception  IllegalArgumentException if lowerBound > upperBound
	 */
	public BoundedRangeGenarator( double lowerBound, double upperBound, 
											NumberGenerator aGenerator ) throws IllegalArgumentException 
		{
		if ( lowerBound > upperBound )
			throw new IllegalArgumentException( "In BoundedRangeGenarator constructor Lower bound: " 
			+ lowerBound + " argument is greater than upperBound: " +  upperBound );
		this.lowerBound = lowerBound;
		this.range = (upperBound - lowerBound);
		this.nestedGenerator = aGenerator;
		}
		
	/**
	 * Returns the next number in a sequence.
	 * The number will be in the interval [lowerBound, upperBound). 
	 */
	public final double nextElement()
		{
		double unboundElement =Math.abs( nestedGenerator.nextElement());
		double offSetBoundedElement =  unboundElement % range ;
		return offSetBoundedElement +  lowerBound;
		}
	}