/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


package sdsu.algorithms.data;

import java.util.Vector;
import java.util.Enumeration;

/**
 * An abstract class defining an interface for probability distributions
 * on a discrete space. 
 * This class and many of its subclasses are adapted from <I>Smalltalk-80: The language
 * and Its Implementation</I>, Goldberg & Robson, Addison-Wesley, 1983, chapter 21, 
 * pages 417-438.
 * @version 1.0 10 June 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public abstract class DiscreteProbability extends ProbabilityDistribution
	{

	/**
	 * Computes the density of each item in the vector.
	 * Returns a vector of the densities.
	 */
	public Vector distribution( Vector sampleSpaceCollection )
		{
		Enumeration dataPoints = sampleSpaceCollection.elements();
		
		Vector theDistribution = new Vector( sampleSpaceCollection.size() );
		while ( dataPoints.hasMoreElements() )
			{
			Number dataPoint =  (Number ) dataPoints.nextElement();
			theDistribution.addElement( new Double(density( dataPoint )) );
			}
		
		return theDistribution;
		}
	}