/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.algorithms.data;

/**
 * Converts a sequence of numbers to a sequence of Integer objects. Uses
 * a NumberGenerator to actually generate the sequence. The class wraps the 
 * number in an Integer object. The numbers generated be the NumberGenerator 
 * are cast to an int.
 *
 * @see		sdsu.data.ObjectGenerator
 * @see		sdsu.data.NumberGenerator
 * @version 1.0 19 June 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */


public class IntegerObjectGenerator implements ObjectGenerator
	{
	
	NumberGenerator sequence;
	
	/**
	 * Create an IntegerObjectGenerator which uses aGenerator
	 * to generate the actual sequence of numbers.
	 */
	public IntegerObjectGenerator( NumberGenerator aGenerator )
		{
		sequence = aGenerator;
		}
	
	/**
	 * Returns the next int in the sequence. The int is rapped in an Integer object.
	 */
	public Object nextElement()
		{
		int nextInt = (int) sequence.nextElement();
		return new Integer( nextInt );
		}
	}