/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


package sdsu.algorithms.data;

/**
 * This class modifies a sequence of numbers. Given a number generator this
 * generator shifts all numbers of the generator by a fixed value.
 *
 * @see		sdsu.data.NumberGenerator
 * @version 1.0 18 June 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */

public class  ShiftedRangeGenarator implements NumberGenerator
	{
	double shift;
	NumberGenerator nestedGenerator;
	
	/**
	 * Creates a generator that shifts the range of its generator. 
	 * @param shift the amount to translate the range aGenerator.
	 */
	public ShiftedRangeGenarator( double shift,  
											NumberGenerator aGenerator ) 
		{
		this.shift = shift;
		this.nestedGenerator = aGenerator;
		}
		
	/**
	 * Returns the next number in a sequence.
	 * The number will be between the lower and upper bound of this generator.
	 */
	public final double nextElement()
		{
		return shift + nestedGenerator.nextElement();
		}
	}