/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.algorithms.data;

/**
 * Generates a sequence of random strings. The characters in the string are random
 * visible ascii characters. The length of the strings is also choosen at random 
 * between a min and max value. 
 *
 * @see		sdsu.data.ObjectGenerator
 * @see		sdsu.data.NumberGenerator
 * @version 1.0 19 June 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */


public class StringGenerator implements ObjectGenerator
	{
	NumberGenerator stringLength;
	NumberGenerator randomCharacter;
	
	/**
	 * Create an StringGenerator which a sequence of random strings.
	 * The strings will be between 1 and maxLength characters in length.
	 */
	public StringGenerator( int maxLength )
		{
		this( 1, maxLength);
		}


	/**
	 * Create an StringGenerator which a sequence of random strings.
	 * The strings will be between minLength and maxLength characters in length.
	 */
	public StringGenerator( int minLength, int maxLength )
		{
		NumberGenerator lengthGenerator = new RandomIntegerGenerator();
		stringLength = new BoundedRangeGenarator( minLength, maxLength + 1 , lengthGenerator );
		
		int asciiSmallA = 97;
		int asciiSmallZ = 122;
		NumberGenerator randomChar = new Zipf( asciiSmallZ - asciiSmallA );
		randomCharacter = new  BoundedRangeGenarator( asciiSmallA - 1, asciiSmallZ, randomChar );
		}
	
	/**
	 * Returns next random string.
	 */
	public Object nextElement()
		{
		int length = (int) stringLength.nextElement();
		StringBuffer newString = new StringBuffer( length );
		for ( int k = 0; k < length; k++ )
			newString.append( (char) randomCharacter.nextElement() );
		return newString.toString();
		}
	}