/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.html;

/**
 * Use the class class to build an HTML form.
 * @version 0.8 6 April 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public class Form implements HTML
	{
	private static final String NEW_LINE = "\n<BR>";
	private static final String POST = "post";
	private static final String GET = "get";
	
	private String url;
	private String method;
	StringBuffer formElements = new StringBuffer();
	
	private Form( String url, String method)
		{
		this.url = url;
		this.method = method;
		}
		
	/**
	 * Returns form object which will send a <B>get</B> request to the given url.
	 * @param url location to send the form when user submits form
	 */
	public static Form methodGet( String url)
		{
		return new Form( url, GET);
		} 
	
	/**
	 * Returns form object which will send a <B>post</B> request to the given url.
	 * @param url location to send the form when user submits form
	 */
	public static Form methodPost( String url)
		{
		return new Form( url, POST);
		}
		 
	/**
	 * Appends the given string to the current end of the form.
	 * Special characters in asciiText are escaped for you.
	 */
	public void append( String asciiText )
		{
		formElements.append( new Formatter( asciiText ) );
		}

	/**
	 * Appends the given HTML object to the current end of the form.
	 * The HTML object is immediately converted to a string, so
	 * any changes made to the object later will not be reflected.
	 */
	public void append( HTML htmlObject )
		{
		formElements.append( htmlObject.toString() );
		}

	/**
	 * Appends the given HTML object then the label to 
	 * the current end of the form.
	 * Special characters in the label are escaped for you.
	 */
	public void append( HTML htmlObject, String label )
		{
		append( htmlObject );
		append( label );
		}

	/**
	 * Appends the given label then the HTML object to 
	 * the current end of the form.
	 * Special characters in the label are escaped for you.
	 */
	public void append( String label, HTML htmlObject )
		{
		append( label );
		append( htmlObject );
		}

	/**
	 * Appends a line break to 
	 * the current end of the form.
	 */
	public void appendLine( )
		{
		formElements.append( NEW_LINE );
		}

	/**
	 * Appends the given HTML object
	 * then line break to 
	 * the current end of the form.
	 */
	public void appendLine( HTML htmlObject )
		{
		append( htmlObject );
		appendLine( );
		}

	/**
	 * Appends the given text
	 * then line break to 
	 * the current end of the form.
	 * Special characters in the label are escaped for you.
	 */
	public void appendLine( String text )
		{
		append( text );
		appendLine( );
		}

	/**
	 * Appends the given label then the HTML object
	 * then line break to 
	 * the current end of the form.
	 * Special characters in the label are escaped for you.
	 */
	public void appendLine( String label, HTML htmlObject )
		{
		append( label );
		append( htmlObject );
		appendLine( );
		}

	/**
	 * Appends the given HTML object then the label
	 * then a line break to 
	 * the current end of the form.
	 * Special characters in the label are escaped for you.
	 */
	public void appendLine( HTML htmlObject, String label )
		{
		append( htmlObject );
		append( label );
		appendLine( );
		}

	/**
	 * Converts the Form to an string with proper html tags
	 */
	public String toString()
		{
		return "\n<FORM ACTION=" + url + 
				" METHOD=" + method + ">" +
				formElements.toString() + "\n</FORM>";
		}
	}
