/*
 * Legalities. This class is based on a class found in 
 * Gary Cornell and Cay S. Horstmann, Core Java. Their class has 6 methods, 
 * this class has 40+ methods. Five of the orginal 6 methods have been completely
 * reimplemented. The following is the notice found in Gary Cornell and Cay S. Horstmann
 * code. Not clear it is needed, but better safe than sorry.
 */
 
/*
 * Gary Cornell and Cay S. Horstmann, Core Java (Book/CD-ROM)
 * Published By SunSoft Press/Prentice-Hall
 * Copyright (C) 1996 Sun Microsystems Inc.
 * All Rights Reserved. ISBN 0-13-565755-5
 *
 * Permission to use, copy, modify, and distribute this 
 * software and its documentation for NON-COMMERCIAL purposes
 * and without fee is hereby granted provided that this 
 * copyright notice appears in all copies. 
 * 
 * THE AUTHORS AND PUBLISHER MAKE NO REPRESENTATIONS OR 
 * WARRANTIES ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. THE AUTHORS
 * AND PUBLISHER SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED 
 * BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING 
 * THIS SOFTWARE OR ITS DERIVATIVES.
 */
 
/* 
 * This covers the SDSU contribution to this class.
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


package sdsu.io;

import java.io.PrintStream;
import java.io.IOException;

/**
 * This class is an extention of the console class found in 
 * Gary Cornell and Cay S. Horstmann, Core Java. The orginal class had 6 methods, this one 
 * has 40+. This class provides simple methods for reading from the stantard input and writing to
 * the standard output. Input is done via statements like 
 * Console.readInt() and Console.readInt( "Prompt string" ). No exceptions are thrown on input. 
 * If there is a read error, the program will exit with a message. If you need to handle exceptions use
 * sdsu.io.ASCIIInputStream.
 * Output is done via Console.print( x ), Console.println( x ) or 
 * Console.print( "x = % .5f\n", x). The later takes a format string like printf in C.
 * The format code has the
 * following structure
 * <ul>
 * <li> a % (required)
 * <li> a modifier (optional)
 * <dl>
 * <dt> + <dd> forces display of + for positive numbers
 * <dt> 0 <dd> show leading zeroes
 * <dt> - <dd> align left in the field
 * <dt> space <dd> prepend a space in front of positive numbers
 * <dt> # <dd> use "alternate" format. Add 0 or 0x for octal or hexadecimal numbers. Don't suppress trailing zeroes in general floating point format.
 * </dl>
 * <li> an integer denoting field width (optional)
 * <li> a period followed by an integer denoting precision (optional)
 * <li> a format descriptor (required)
 * <dl>
 * <dt>f <dd> floating point number in fixed format
 * <dt>e, E <dd> floating point number in exponential notation (scientific format). The E format results in an uppercase E for the exponent (1.14130E+003), the e format in a lowercase e.
 * <dt>g, G <dd> floating point number in general format (fixed format for small numbers, exponential format for large numbers). Trailing zeroes are suppressed. The G format results in an uppercase E for the exponent (if any), the g format in a lowercase e.
 * <dt>d, i <dd> integer in decimal
 * <dt>x <dd> integer in hexadecimal
 * <dt>o <dd> integer in octal
 * <dt>s <dd> string
 * <dt>c <dd> character
 * </dl>
 * </ul>
 *
 * @version 1.1 14 Sept 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */

public class Console
	{
	private static ASCIIInputStream input = new ASCIIInputStream( System.in );
	
    /**
     * Example uses of this class
     */
	public static void main( String[] args)
		{
		// Simple input
		
		Console.print( "Print an integer ");
		int why = Console.readInt();
		Console.println( "You typed: " + why );
		
		int easy = Console.readInt( "Print another integer" );
		Console.print( "You typed: %20i\n ", easy );

		
		String message = Console.readLine( "Type a line of text" );
		// Simple printing
		Console.println( 5 );
		Console.print( "Hi Mom" );
		Console.println( " Hi Dad" );
		
		// Sample use of the formating feature of print statement
		double x = 1.23456789012;
		double y = 123;
		double z = 1.2345e30;
		double w = 1.02;
		double u = 1.234e-5;
		int d = 0xCAFE;
		Console.print( "x = |%f|\n", x);
		Console.print( "u = |%20f|\n", u);
		Console.print( "x = |% .5f|\n", x);
		Console.print( "w = |%20.5f|\n", w);
		Console.print( "x = |%020.5f|\n", x);
		Console.print( "x = |%+20.5f|\n", x);
		Console.print( "x = |%+020.5f|\n", x);
		Console.print( "x = |% 020.5f|\n", x);
		Console.print( "y = |%#+20.5f|\n", y);
		Console.print( "y = |%-+20.5f|\n", y);
		Console.print( "z = |%20.5f|\n", z);
		}

    /**
     * Exit programs after displaying a message and an Exception
     * @param	error	Exception to be displayed
     * @param	messge	message to be displayed
     */
	private static void exitProgram( Exception error, String message )
		{
		System.err.println( message );
		System.err.println( error.getMessage() );
		System.exit(1);		
		}

    /**
     * Flushes the print stream and returns whether or not there was
     * an error on the output stream.  Errors are cumulative; once the
     * print stream encounters an error this routine will continue to
     * return true on all successive calls.
     * @return true if the print stream has ever encountered an error
     * on the output stream.
     */
 	public static boolean checkError()
 		{
 		return System.out.checkError();
 		}
 	
    /**
     * Forces the contents of the ouput buffer to be displayed on standard out.
     * Buffer then emptied.
     */
	public static void flush()
		{
		System.out.flush();
		}

	
    /**
     * Displays standard out a string prompt followed by a space
     * @param	prompt	string displayed on standard out
     */
	protected static void printPrompt( String prompt )
		{  
		System.out.print(prompt + " ");
      	System.out.flush();
		}

    /**
     * Prints a boolean and insures that the  
     * contents of output buffer is printed on standard out.
     * @param value the boolean to be printed
     */
	public static void print( boolean value )
		{
		System.out.print( value );
		flush();
		}
	
    /**
     * Prints a char and insures that the  
     * contents of output buffer is printed on standard out.
     * @param value the char to be printed
     */
	public static void print( char value )
		{
		System.out.print( value );
		flush();
		}
	
    /**
     * Prints an array of characters and insures that the  
     * contents of output buffer is printed on standard out.
     * @param value the array of character to be printed
     */
	public static void print( char[] value )
		{
		System.out.print( value );
		flush();
		}
	
    /**
     * Prints a double and insures that the  
     * contents of output buffer is printed on standard out.
     * @param value the double to be printed
     */
	public static void print( double value )
		{
		System.out.print( value );
		flush();
		}
	
    /**
     * Prints a float and insures that the  
     * contents of output buffer is printed on standard out.
     * @param value the float to be printed
     */
	public static void print( float value )
		{
		System.out.print( value );
		flush();
		}
	
    /**
     * Prints a integer and insures that the  
     * contents of output buffer is printed on standard out.
     * @param value the integer to be printed
     */
	public static void print( int value )
		{
		System.out.print( value );
		flush();
		}
	
    /**
     * Prints a long integer and insures that the  
     * contents of output buffer is printed on standard out.
     * @param value the long to be printed
     */
	public static void print( long value )
		{
		System.out.print( value );
		flush();
		}
	
    /**
     * Prints an object and insures that the  
     * contents of output buffer is printed on standard out.
     * @param value the object to be printed
     */
	public static void print( Object value )
		{
		System.out.print( value );
		flush();
		}
	
    /**
     * Prints a string and insures that the  
     * contents of output buffer is printed on standard out.
     * @param value the string to be printed
     */
	public static void print( String value )
		{
		System.out.print( value );
		flush();
		}

	/** 
	 * prints a character following printf conventions
	 * @param format the format string
	 * @param value the character to print
	 */
	public static void print( String format, char value )
		{
		try
			{
			Format.print( System.out, format, value );
			flush();
			}
		catch ( IllegalArgumentException error )
			{
			exitProgram( error, "Format problem with " + format );
			}
		}

	/** 
	 * prints a formatted double (and float) following printf conventions
	 * @param format the format string
	 * @param value the double to print
	 */
	public static void print( String format, double value )
		{
		try
			{
			Format.print( System.out, format, value );
			flush();
			}
		catch ( IllegalArgumentException error )
			{
			exitProgram( error, "Format problem with " + format );
			}
		}
	
	/** 
	 * prints a formatted long integer (and integer) following printf conventions
	 * @param format the format string
	 * @param value the double to print
	 */
	public static void print( String format, long value )
		{
		try
			{
			Format.print( System.out, format, value );
			flush();
			}
		catch ( IllegalArgumentException error )
			{
			exitProgram( error, "Format problem with " + format );
			}
		}

	/** 
	 * prints a formatted string following printf conventions
	 * @param format the format string
	 * @param value the string to print
	 */
	public static void print( String format, String value )
		{
		try
			{
			Format.print( System.out, format, value );
			flush();
			}
		catch ( IllegalArgumentException error )
			{
			exitProgram( error, "Format problem with " + format );
			}
		}

    /**
     * Prints a newline then insures that the 
     * contents of output buffer is printed on standard out.
     */
    public static void println() 
    	{
		System.out.println(  );
		flush();
    	}

    /**
     * Prints a boolean followed by a newline then insures that the 
     * contents of output buffer is printed on standard out.
     * @param value the boolean to be printed
     */
	public static void println( boolean value )
		{
		System.out.println( value );
		flush();
		}
	
    /**
     * Prints a character followed by a newline then insures that the  
     * contents of output buffer is printed on standard out.
     * @param value the character to be printed
     */
	public static void println( char value )
		{
		System.out.println( value );
		flush();
		}
	
	
    /**
     * Prints a double followed by a newline then insures that the  
     * contents of output buffer is printed on standard out.
     * @param value the double to be printed
     */
	public static void println( double value )
		{
		System.out.println( value );
		flush();
		}
	
    /**
     * Prints a float followed by a newline then insures that the  
     * contents of output buffer is printed on standard out.
     * @param value the float to be printed
     */
	public static void println( float value )
		{
		System.out.println( value );
		flush();
		}
	
    /**
     * Prints a integer followed by a newline then insures that the  
     * contents of output buffer is printed on standard out.
     * @param value the integer to be printed
     */
	public static void println( int value )
		{
		System.out.println( value );
		flush();
		}
	
    /**
     * Prints a long integer followed by a newline then insures that the  
     * contents of output buffer is printed on standard out.
     * @param value the long integer to be printed
     */
	public static void println( long value )
		{
		System.out.println( value );
		flush();
		}
	
    /**
     * Prints a object followed by a newline then insures that the  
     * contents of output buffer is printed on standard out.
     * @param value the object to be printed
     */
	public static void println( Object value )
		{
		System.out.println( value );
		flush();
		}
	
    /**
     * Prints a string followed by a newline then insures that the  
     * contents of output buffer is printed on standard out.
     * @param value the string to be printed
     */
	public static void println( String value )
		{
		System.out.println( value );
		flush();
		}
 	
	// Input methods ----------------------------------------------------
    /**
 	 * Do not use. This method no longer does anything meaningful. Always returns false. 
 	 * @deprecated
     */
	public static boolean bad()
		{
		return input.bad();
		} 

    /**
     * Determines if stream is has reach end of file.
     * Consumes white space up to next token.
     * @return <i>true</i> if end of file was reached or if only 
     *	white space remained in the stream. 
     */
	public static boolean eof()
		{
		try
			{
			return input.eof();
			}
		catch (IOException error )
			{
			exitProgram( error, "IO error checking for end of file"  );
			} 
			// Unreachable code, but method must end in return or exception 
			return false;
		}
		
    /**
     * Causes the next I/O operation to start at the beginning of the next
     * input line.  Lines are delimited by either <i>cr</i>, <i>cr lf</i>,
     * or <i>lf</i>.
     */
	public static void flushLine() 
		{
		try
			{
			input.flushLine();
			}
		catch ( Exception error )
			{
			exitProgram( error, "IO error in flushLine"  );
			}
		}
	
    /**
     * Reads an ASCII boolean value.
     * It reads a word and determines if it represents <i>true</i> or
     * <i>false</i>.  Possible values are: <i>true</i> and <i>false</i>.
     * The comparison is case insensitive.
     * Program exits on read errors
     * @return the boolean.
     */
	public static boolean readBoolean() 
		{
		try
			{
			return input.readBoolean();
			}
		catch ( Exception error )
			{
			exitProgram( error, "Read error in readBoolean"  );
			
			// Unreachable code, but method must end in return or exception 
			return false;
			}
		}

	/**
     * Reads an ASCII boolean value, after displaying given prompt.
     * It reads a word and determines if it represents <i>true</i> or
     * <i>false</i>.  Possible values are: <i>true</i> and <i>false</i>.
     * The comparison is case insensitive.
     * The eof() method will return true after this method has attempted
     * to read beyond the end of file.
     * Program exits on read errors
	 * @param prompt the prompt string to display
     * @return the boolean.
     */
	public static boolean readBoolean( String prompt ) 
		{
		printPrompt( prompt );
		return readBoolean();
		}


    /**
     * Read an ASCII character and convert it into the internal <b>char</b>
     * format.
     * The eof() method will return true after this method has attempted
     * to read beyond the end of file.
     * Program exits on read errors
     * @return the character.
     */
	public static char readChar() 
		{
		try
			{
			return input.readChar();
			}
		catch ( Exception error )
			{
			exitProgram( error, "Read error in readChar"  );
			
			// Unreachable code, but method must end in return or exception 
			return 'a';
			}
		}

    /**
     * Read an ASCII character and convert it into the internal <b>char</b>
     * format, after displaying the given prompt.
     * The eof() method will return true after this method has attempted
     * to read beyond the end of file.
     * Program exits on read errors
	 * @param prompt the prompt string to display
     * @return the character.
     */
	public static char readChar( String prompt ) 
		{
		printPrompt( prompt );
		return readChar();
		}

    /**
     * Reads an ASCII decimal floating point number.
     * A floating point number is defined as follows:
     * <ul><li>an optional '-' to make the number negative</li>
     * <li>0 or more digits</li>
     * <li>an optional period follows by more digits</li>
     * <li>an optional 'e' or 'E' to introduce the exponent<ul>
     *   <li>an optional '-' to make the exponent negative</li>
     *   <li>digits making up the exponent</li>
     * </ul></li></ul>
     * Doubles can be preceded by optional whitespace.
     * whitespace is defined as SPACE, TAB, CR, or NL.<br>
     * Program exits on read errors
     * @return the double.
     */
	public static double readDouble() 
		{
		try
			{
			return input.readDouble();
			}
		catch ( Exception error )
			{
			exitProgram( error, "Read error in readDouble"  );
			
			// Unreachable code, but method must end in return or exception 
			return 0.0;
			}
		}

    /**
     * Reads an ASCII decimal floating point number, after displaying given prompt.
     * A floating point number is defined as follows:
     * <ul><li>an optional '-' to make the number negative</li>
     * <li>0 or more digits</li>
     * <li>an optional period follows by more digits</li>
     * <li>an optional 'e' or 'E' to introduce the exponent<ul>
     *   <li>an optional '-' to make the exponent negative</li>
     *   <li>digits making up the exponent</li>
     * </ul></li></ul>
     * Doubles can be preceded by optional whitespace.
     * whitespace is defined as SPACE, TAB, CR, or NL.<br>
     * Program exits on read errors
     * @return the double.
	 * @param prompt the prompt string to display
     */
	public static double readDouble( String prompt ) 
		{
		printPrompt( prompt );
		return readDouble();
		}


   /**
     * Reads an ASCII decimal floating point number.
     * A floating point number is defined as follows:
     * <ul><li>an optional '-' to make the number negative</li>
     * <li>0 or more digits</li>
     * <li>an optional period follows by more digits</li>
     * <li>an optional 'e' or 'E' to introduce the exponent<ul>
     *   <li>an optional '-' to make the exponent negative</li>
     *   <li>digits making up the exponent</li>
     * </ul></li></ul>
     * Floats can be preceded by optional whitespace.
     * whitespace is defined as SPACE, TAB, CR, or NL.<br>
     * Program exits on read errors
     * @return the float.
     */
	public static float readFloat() 
		{
		try
			{
			return input.readFloat();
			}
		catch ( Exception error )
			{
			exitProgram( error, "Read error in readFloat"  );
			
			// Unreachable code, but method must end in return or exception 
			return 0.0F;
			}
		}

   /**
     * Reads an ASCII decimal floating point number, after displaying given prompt.
     * A floating point number is defined as follows:
     * <ul><li>an optional '-' to make the number negative</li>
     * <li>0 or more digits</li>
     * <li>an optional period follows by more digits</li>
     * <li>an optional 'e' or 'E' to introduce the exponent<ul>
     *   <li>an optional '-' to make the exponent negative</li>
     *   <li>digits making up the exponent</li>
     * </ul></li></ul>
     * Floats can be preceded by optional whitespace.
     * whitespace is defined as SPACE, TAB, CR, or NL.<br>
     * Program exits on read errors
     * @return the float.
	 * @param prompt the prompt string to display
     */
	public static float readFloat( String prompt ) 
		{
		printPrompt( prompt );
		return readFloat();
		}

    /**
     * Reads an ASCII decimal integer.
     * Integers can be preceded by optional whitespace.
     * whitespace is defined as SPACE, TAB, CR, or NL.
     * The eof() method will return true after this method has attempted
     * to read beyond the end of file.
     * Program exits on read errors
     * @return the integer.
     */
	public static int readInt() 
		{
		try
			{
			return input.readInt();
			}
		catch ( Exception error )
			{
			exitProgram( error, "Read error in readInt"  );
			
			// Unreachable code, but method must end in return or exception 
			return 0;
			}
		}

    /**
     * Reads an ASCII decimal integer, after displaying given prompt.
     * Integers can be preceded by optional whitespace.
     * whitespace is defined as SPACE, TAB, CR, or NL.
     * The eof() method will return true after this method has attempted
     * to read beyond the end of file.
     * Program exits on read errors
	 * @param prompt the prompt string to display
     * @return the integer.
     */
	public static int readInt( String prompt ) 
		{
		printPrompt( prompt );
		return readInt();
		}

	
    /**
     * Reads in a line that has been terminated by a \n, \r, 
     * \r\n or EOF.
     * The eof() method will return true if the
     * read attempted to go beyond the end of file.
     * The terminating line characters will not be part of the String
     * that is returned.
     * Program exits on read errors
     * @return a String copy of the line or null if nothing more could be read.
     */
	public static String readLine() 
		{
		try
			{
			return  input.readLine();
			}
		catch ( Exception error )
			{
			exitProgram( error, "Read error in readInt"  );
			
			// Unreachable code, but method must end in return or exception 
			return "";
			}
		}

    /**
     * Reads in a line that has been terminated by a \n, \r, 
     * \r\n or EOF, after displaying the given prompt.
     * The eof() method will return true if the
     * read attempted to go beyond the end of file.
     * The terminating line characters will not be part of the String
     * that is returned.
     * Program exits on read errors
	 * @param prompt the prompt string to display
     * @return a String copy of the line or null if nothing more could be read.
     */
	public static String readLine( String prompt ) 
		{
		printPrompt( prompt );
		return readLine();
		}

    /**
     * Reads an ASCII decimal long.
     * Longs can be preceded by optional whitespace.
     * whitespace is defined as SPACE, TAB, CR, or NL.
     * The eof() method will return true after this method has attempted
     * to read beyond the end of file.
     * Program exits on read errors
     * @return the long or 0 when EOF was reached.
     */
	public static long readLong() 
		{
		try
			{
			return input.readLong();
			}
		catch ( Exception error )
			{
			exitProgram( error, "Read error in readLong"  );
			
			// Unreachable code, but method must end in return or exception 
			return 0L;
			}
		}

    /**
     * Reads an ASCII decimal long, after displaying the given prompt.
     * Longs can be preceded by optional whitespace.
     * whitespace is defined as SPACE, TAB, CR, or NL.
     * The eof() method will return true after this method has attempted
     * to read beyond the end of file.
     * Program exits on read errors
	 * @param prompt the prompt string to display
     * @return the long or 0 when EOF was reached.
     */
	public static long readLong( String prompt ) 
		{
		printPrompt( prompt );
		return readLong();
		}


    /**
     * Reads an ASCII decimal short.
     * Shorts can be preceded by optional whitespace.
     * whitespace is defined as SPACE, TAB, CR, or NL.
     * The eof() method will return true after this method has attempted
     * to read beyond the end of file.
     * Program exits on read errors
     * @return the short.
     * @see ASCIIInputStream@readInt
     */
	public static short readShort() 
		{
		try
			{
			return input.readShort();
			}
		catch ( Exception error )
			{
			exitProgram( error, "Read error in readShort"  );
			
			// Unreachable code, but method must end in return or exception 
			return (short) 0;
			}
		}

    /**
     * Reads an ASCII decimal short, after displaying given prompt.
     * Shorts can be preceded by optional whitespace.
     * whitespace is defined as SPACE, TAB, CR, or NL.
     * The eof() method will return true after this method has attempted
     * to read beyond the end of file.
     * Program exits on read errors
	 * @param prompt the prompt string to display
     * @return the short.
     * @see ASCIIInputStream@readInt
     */
	public static short readShort( String prompt ) 
		{
		printPrompt( prompt );
		return readShort();
		}

    /**
     * Reads a word.
     * A word is a string of characters deliminated by whitespace or
     * EOF.
     * A word can be preceded by optional whitespace characters.
     * Whitespace is defined as SPACE, TAB, CR, or NL.<br>
     * Program exits on read errors
     * @return the word or null if EOF reached.
     */
	public static String readWord() 
		{
		try
			{
			return input.readWord();
			}
		catch ( Exception error )
			{
			exitProgram( error, "Read error in readWord"  );
			
			// Unreachable code, but method must end in return or exception 
			return "";
			}
		}

    /**
     * Reads a word, after displaying given prompt.
     * A word is a string of characters deliminated by whitespace or
     * EOF.
     * A word can be preceded by optional whitespace characters.
     * Whitespace is defined as SPACE, TAB, CR, or NL.<br>
     * Program exits on read errors
	 * @param prompt the prompt string to display
     * @return the word or null if EOF reached.
     */
	public static String readWord( String prompt ) 
		{
		printPrompt( prompt );
		return readWord();
		}	
	}