/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.io;

import java.io.FilterReader;
import java.io.Reader;
import java.io.IOException;
import java.io.InputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;

import sdsu.util.*;

/**
 * Allows you to read to the EOF with one method. Useful in reading 
 * the entire contents of a file.
 * @version 1 25 August 1997
 * @author Roger Whitney 
 *   (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */

public class SimpleReader extends FilterReader implements Cloneable
	{
	protected static int EOFIndicator = -1;
	private int readSize = 512;
		
	public SimpleReader( Reader in  )
		{
		super( in );
		}

	public SimpleReader( InputStream in  )
		{
		this( new BufferedReader(new InputStreamReader(in)) );
		}

	/**
	 * Returns the entire contents (to EOF) of the input stream.
	 */
	public String contents() throws IOException
		{
		StringBuffer contents = new StringBuffer( readSize * 10 );
		
		char[] filteredChars = new char[readSize];
		int charsRead = read( filteredChars, 0, readSize);
		
		while ( charsRead != EOFIndicator)
			{
			contents.append( filteredChars, 0, charsRead );
			charsRead = read( filteredChars, 0, readSize);
			}
			return contents.toString();
		}			
	
	/**
	 * Set the number of characters that will be read at one time by
	 * contents. Larger values reduce the number of reads required.
	 * Default value is 512.
	 */
	public void setReadSize( int readSize)
		{
		this.readSize = readSize;
		}		
	} // end class
