/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.io;


import java.io.OutputStream;

/**
 * An output stream that collects it output in a stringbuffer, which can be used
 * to create a string.
 *
 * @version 1.0 28 November 1997
 * @author Roger Whitney 
 *   (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 * @see java.io.StringWriter
 */ 
public class StringOutputStream extends OutputStream
	{
	StringBuffer output;
	
	/**
	 * Create a new StringOutputStream with the default initial buffer size.
	 */
	public StringOutputStream()
		{
		output = new StringBuffer( );
		}

	/**
	 * Create a new StringOutputStream with the specified initial buffer size.
	 */
	public StringOutputStream(int initialBufferSize)
		{
		output = new StringBuffer( initialBufferSize );
		}
	
	/**
	 * Close the stream
	 */
	public void close()
		{
		// nothing to close
		}
	
	/** 
	 * Flush the stream
	 */
	public void flush()
		{
		// nothing to flush
		}
	
	/**
	 * Return the stringbuffer
	 */
	public StringBuffer getBuffer()
		{
		return output;
		}
	
	/**
	 * Return the output as a String.
	 */
	public String toString()
		{
		return output.toString();
		}
	
	/**
	 * Write a single character. Ignores end of file.
	 */
	public void write( int outputChar)
		{
		if (outputChar != -1 )
			output.append( (char) outputChar );
		}
	}