/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.logging;

import java.io.Serializable;

/**
 * LoggerContext and its subclasses are used by sdsu.logging.Logger. 
 * Contexts allow storage of information between individual messages.
 * Java does not lend it self to using context as used in the orginal
 * article listed below.
 * <P> The logging system is based on the logging pattern by 
 * Neil Harrison in <I>Pattern Languages of Program Design 3</I>
 * Eds Martin, Riehle, Buschman, 1998, pp 277-289
 *
 * @see sdsu.logging.Logger
 * @see sdsu.logging.LoggerMessage
 * @version 1.0 11 January 1998 
 * @author Roger Whitney 
 * (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public abstract class LoggerContext
	implements Serializable
	{
	
	// The following are used in converting message to a 
	// string containing key-value pairs
	protected static final String PAIR_SEPARATOR = ";";
	protected static final String KEY_VALUE_SEPARATOR = "=";


	LoggerMessage message;


	/**
	 * Add a message to the context
	 */
	public void setMessage( LoggerMessage newMessage )
		{
		message = newMessage;
		}
		
	/**
	 * Displays a string to be displayed on the screen for this 
	 * context.
	 */
	public String displayString()
		{
		return message.displayString();
		}
		
	/**
	 * Displays a string to be saved in a log for this 
	 * context.
	 */
	public String logString()
		{
		return message.logString();
		}

	}