/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.logging;

 /**
 * This logger ignores all log requests. Useful in turning off logging
 * without removing all loging statements.
 *
 * @see sdsu.logging.Logger
 * @version 1.1 18 January 1998 
 * @author Roger Whitney 
 * (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
final public class NullLogger extends LoggerImplementation
	{
	
	/**
	 * Create a NullLogger and register it with Logger
	 * Returns the current active logger. If a logger has
	 * previously been set, it will be that logger.
	 */
	public static LoggerImplementation register( )
		{
		if ( !Logger.hasLogger() )
			{
			NullLogger instance = new NullLogger( ); 
			Logger.register( instance );
			}
		return Logger.getLogger();;
		}

	NullLogger()
		{
		}
		
	/**
	 * Null body ignores log request
	 */
	protected void processLogRequest( LoggerMessage message )
		{
		}

	}