/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.logging;

 /**
 * This logger directs all log messages to System.err, which normally
 * is the terminal. As a SelectiveLogger, message types can be turned 
 * off.
 *
 * @see sdsu.logging.Logger
 * @see sdsu.logging.SelectiveLogger
 * @version 1.1 18 January 1998 
 * @author Roger Whitney 
 * (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public class ScreenLogger extends SelectiveLogger
	{
	
	/**
	 * Create a ScreenLogger and register it with Logger.
	 * Returns the current active logger. If a logger has
	 * previously been set, it will be that logger.
	 */
	public static LoggerImplementation register( )
		{
		if ( !Logger.hasLogger() )
			{
			ScreenLogger instance = new ScreenLogger( ); 
			Logger.register( instance );
			}
		return Logger.getLogger();;
		}

	ScreenLogger()
		{
		}
	
	/**
	 * Add the LogMessage object to the log. 
	 */
	protected void processLogRequest( LoggerMessage message )
		{
		System.err.println( message.displayString() );
		System.err.flush();
		}
	
	}