/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.logging;

import java.util.Vector;

 /**
 * SelectiveLogger allows you to turn off logging for a
 * type of message. Default activity is logging turn on
 * for all types of messages.
 *
 * @see sdsu.logging.Logger
 * @version 1.1 18 January 1998 
 * @author Roger Whitney 
 * (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public abstract class SelectiveLogger extends LoggerImplementation
	{
	Vector messagesTurnedOff = new Vector();
	
	/**
	 * Turns logging off for messages of the given type.
	 */
	public void messageTypeOff( String messageType )
		{
		messagesTurnedOff.addElement( messageType );
		}

	/**
	 * Turns logging on for messages of the given type.
	 */
	public void messageTypeOn( String messageType )
		{
		messagesTurnedOff.removeElement( messageType );
		}
		
	/**
	 * Turns logging of debug statements on.
	 */
	public void debugOn()
		{
		messageTypeOn( new DebugMessage("",1).getMessageType() );
		}

	/**
	 * Turns logging of error statements on.
	 */
	public void errorOn()
		{
		messageTypeOn( new ErrorMessage("").getMessageType() );
		}
		
	/**
	 * Turns logging of warning statements on.
	 */
	public void warningOn()
		{
		messageTypeOn( new WarningMessage("").getMessageType() );
		}

	/**
	 * Turns logging of log statements on.
	 */
	public void logOn()
		{
		messageTypeOn( new LogMessage("").getMessageType() );
		}

	/**
	 * Turns logging of debug statements off.
	 */
	public void debugOff()
		{
		messageTypeOff( new DebugMessage("", 1).getMessageType() );
		}

	/**
	 * Turns logging of error statements off.
	 */
	public void errorOff()
		{
		messageTypeOff( new ErrorMessage("").getMessageType() );
		}
		
	/**
	 * Turns logging of warning statements off.
	 */
	public void warningOff()
		{
		messageTypeOff( new WarningMessage("").getMessageType() );
		}

	/**
	 * Turns logging of log statements off.
	 */
	public void logOff()
		{
		messageTypeOff( new LogMessage("").getMessageType() );
		}


	/**
	 * Add the LogMessage object to the log. Subclasses must
	 * implement.
	 */
	public void log( LoggerMessage message )
		{
		if (!messagesTurnedOff.contains( message.getMessageType()))
			processLogRequest( message);
		}

	}