/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.rmi.registry;

import java.rmi.*;

/**
 * Interface implemented by UniVMRegistry and used by
 * Registrar to add objects to the UniVMRegistry in the 
 * UniVMRegistry's VM. There should be no reason to use
 * this interface directly. Use Registrar to add objects
 * to the UniVMRegistry.
 *
 * @see sdsu.rmi.registry.Registrar
 * @version 0.9.1 2 Feb. 1998 
 * @author Roger Whitney 
 * (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */

public interface RemoteRegistrar extends java.rmi.Remote 
	{

	/**
	 * Binds the name to the specified remote object.
	 * @exception RemoteException If remote operation failed.
	 * @exception AlreadyBoundException If name is already bound.
	 */
	public void bind(String name, Remote object)
		throws RemoteException, AlreadyBoundException;

	/**
	 * Binds a Remote object created from the class className in the
	 * rmiregistry. 
	 * The registry's classpath <B>must</B> include the className.
	 * The registry's classpath <B>must</B> access the class given
	 * by the className.
	 * @param className Full class name, used to create remote object.
	 *		<B>Class must implement java.rmi.Remote.
	 * @param name Name to which object is bound
	 * @returns Name to which object is bound to.  
	 * @exception RemoteException thrown if there is any problem
	 * 	binding object.
	 * @exception AlreadyBoundException If name is already bound.
	 */
	public void bind(String label, String className) 
		throws RemoteException, AlreadyBoundException;

	/**
	 * Binds a Remote object created from the class className in the
	 * rmiregistry. Replaces and object currently bound at same name.
	 * The registry's classpath <B>must</B> include the className.
	 * The registry's classpath <B>must</B> access the class given
	 * by the className.
	 * @param className Full class name, used to create remote object.
	 *		<B>Class must implement java.rmi.Remote.
	 * @param name Name to which object is bound
	 * @returns Name to which object is bound to.  
	 * @exception RemoteException thrown if there is any problem
	 * 	binding object.
	 */
	public void rebind(String label, String className) 
		throws java.rmi.RemoteException;

	/**
	 * Binds a Remote object created from the class className in the
	 * rmiregistry. Replaces and object currently bound at same name.
	 * The registry's classpath <B>must</B> access the class given
	 * by the className.
	 * @param object  Remote object which is bound.
	 * @param name Name to which object is bound.
	 * @returns Name to which object is bound to.  
	 * @exception RemoteException thrown if there is any problem
	 * 	binding object.
	 */
	public void rebind( String name, Remote object) 
		throws RemoteException;
	
	/**
	 * Returns an array of the names in the registry.
	 * @exception RemoteException If remote operation failed.
	 * @exception AccessException If this operation is not permitted.
	 */
	public String[] list() throws  RemoteException, AccessException;

	}