/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


package sdsu.test;

/**
 * This class is modeled after the ASSERT macro in C/C++. The class 
 * has four static methods. 
 * The methods are identical. Each method will throw an exception if
 * is arguement is false. A sample use of the assert is:
 * <pre>
 *		Asset.condition( (X > 5 ) && ( Z <= 10 ));
 * </pre>
 * This asserts that the condition should be true at this point.
 * If it is not an exception will be thrown. So the program can
 * assume that the condition is true after the above assert. <P>
 * In C/C++ ASSERT is defined as a macro. Thus the macro can be 
 * turned off with a compile switch. This allows the code to contain
 * ASSERTS with out any runtime cost. There is no way in Java to
 * completely eliminate the cost of the Assert calls. Hopefully
 * future compilers will be smart enough to use tricks with 
 * classpaths and two versions of this class to "turn off" the
 * Assert calls in productin code without modifing the program
 * source.<P>
 * See <I>Writing Solid Code</I> by Steve Maguire, pp 13-44
 * or <I>Object-Oriented Software Construction</I> by 
 * Bertrand Meyer, pp 111 - 163 for the use and importance
 * of Assert.
 *
 * @see sdsu.test.AssertException
 * @version 1.0 30 December 1997 
 * @author Roger Whitney 
 * (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public final class Assert
	{		
	/**
	 * Throws AssertException if result is false.
	 * AssertException is runtime exception, when you are not
	 * forced to catch it.
	 */
	public static void condition( boolean result )
		{
		 if ( !result )
		 	throw new AssertException( "False assert condition" );
		}

	/**
	 * Throws AssertException if result is false.
	 * AssertException is runtime exception, when you are not
	 * forced to catch it.
	 */
	public static void precondition( boolean result )
		{
		 if ( !result )
		 	throw new AssertException( "False assert precondition" );
		}

	/**
	 * Throws AssertException if result is false.
	 * AssertException is runtime exception, when you are not
	 * forced to catch it.
	 */
	public static void postcondition( boolean result )
		{
		 if ( !result )
		 	throw new AssertException( "False assert postcondition" );
		}

	/**
	 * Throws AssertException if result is false.
	 * AssertException is runtime exception, when you are not
	 * forced to catch it.
	 */
	public static void classInvariant( boolean result )
		{
		 if ( !result )
		 	throw new AssertException( "False assert classInvariant" );
		}

	}