/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.test;

import sdsu.io.Console;
import java.util.Enumeration;

/**
 * This Properties acts like a 
 * normal hashtable or Properties, except when a key or value is not found on a lookup. In this case, the hashtable 
 * prompts the user via standard in for the value in question. 
 * <strong>This class should only be used for testing programs.</strong>
 * This class is designed to aid in testing java programs that are run as cgi programs.
 * When run as a cgi program, various information is sent to the program. When the program is
 * tested outside the cgi environment, that data is not available. 
 * @version 1.0 2 April 1997 
 * @author Roger Whitney (<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */
public class InteractiveProperties extends java.util.Properties
	{
	/**
	 * Returns true if hashtable cantains value as an element.
	 * If the table does not contain the value, the user is prompted for the correct
	 * answer.
	 */ 
	public synchronized boolean contains(Object value)
		{
		if (super.contains( value ) )	
			return true;
			
		return Console.readBoolean( "Does hashtable contain " + 
				value.toString() + "? Type yes or no followed by a return" );
		}

	/**
    * Returns true if the collection contains an element for the key.
	 * If the table does not contain the value, the user is prompted for the correct
	 * answer.
    * @param key the key that we are looking for
	 */ 
	public synchronized boolean containsKey(Object key)
		{
		if (super.containsKey( key ) )	
			return true;
		
		return valueFromUser( key);
		}

    /**
     * Gets the object associated with the specified key in the 
     * hashtable. If table does not contain the key, prompt the user for 
     * if the table should have the value.
     * @param key the specified key
     * @returns the element for the key or null if the key
     * 		is not defined in the hash table.
     * @see Hashtable#put
     */
    public synchronized Object get(Object key) 
    	{
		if (super.containsKey( key ) )	
			return super.get( key);
			
    	boolean contains  = valueFromUser( key );
    	if ( contains )
    		return  super.get( key);
    	else
    		return null;
		}

	/**
	 * Gets a property with the specified key. If the key is not 
	 * found in this property list, tries the defaults. This method 
    * returns null if the property is not found.
    * @param key the hashtable key
    */
	public String getProperty(String key) 
		{
		// Copied code from Properties class, except call local get not hashtable get
		String val = (String) get(key);
		return ((val == null) && (defaults != null)) ? defaults.getProperty(key) : val;
		}

    /**
     * Returns an enumeration of the hashtable's keys.
     * @see Hashtable#elements
     * @see Enumeration
     */
    public synchronized Enumeration keys() 
    	{
    	Enumeration currentKeys = super.keys();
    	Console.println( "The hashtable currently has the following keys: ");
    	
    	while (currentKeys.hasMoreElements() )
    		Console.println( currentKeys.nextElement() );
    	
    	int missingKeys = Console.readInt( "How many more keys should be added ");
    	
    	if (missingKeys > 0 )
    		{
    		Console.println( "Type the key and its value separated by a space");
    		for (int k = 1; k <= missingKeys; k++ )
    			{
    			Console.print( "Pair: " + k + " " );
    			String key = Console.readWord();
    			String value = Console.readWord();
    			put( key, value);
    			}
    		}
		return super.keys();
		}
	
	/**
	 * Gets value for key from user. Value and key are stored in table.
	 * Returns true if user give value, false if no value is given
	 */
	 protected boolean valueFromUser( Object key )
	 	{
	 	boolean contains = Console.readBoolean( "Should hashtable contain key " + 
				key.toString() + "? Type yes or no followed by a return" );
		
		if ( contains )
			{
	 		String value = Console.readLine( "Type value for key: " + key + " " );
			put( key, value);
			}
		return contains;
	 	}
	}