/* 
 * Copyright (C) 1997 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


package sdsu.util;

import java.util.Vector;
import java.util.Enumeration;
import sdsu.compare.*;

/**
 * SortedEnumeration is an enumerations that returns elements in 
 * sorted order. Requires that all elements of the source enumeration be
 * of the type expected by the comparer provided to the constructor.
 */
public class SortedEnumeration implements Enumeration
	{
	Enumeration orderedElements;
	
	/**
	 * Creates a new SortedEnumeration. The elements of sourceEnumeration
	 * are return in sorted order indicated by order.
	 * @exception ClassCastException If all elements of sourceEnumeration are not 
	 * the type expected by the comparer <B>order</B> .
	 *  ClassCastException is a RuntimeException, so compiler does not force you
	 *  to catch this exception.
	 */
	public SortedEnumeration( Enumeration sourceEnumeration, Comparer order)
		{
		Vector elements = new Vector();
		
		// For efficiency collect all elements, then sort them
		while ( sourceEnumeration.hasMoreElements() )
			elements.addElement( sourceEnumeration.nextElement() );
		
		SortedList sorter = new SortedList( order );
		sorter.addElements( elements );
		
		orderedElements = sorter.elements();
		}
	
	/**
	 * Returns true if the enumeration has more elements
	 */
	public boolean hasMoreElements()
		{
		return orderedElements.hasMoreElements();
		}
	
	/**
	 * Returns the next element in the enumeration
	 */
	public Object nextElement()
		{
		return orderedElements.nextElement();
		}
	
	/**
	 * Requires that the elements of sourceEnumeration be strings.
	 * Returns an enumeration with the elements of sourceEnumeration in 
	 * increasing sorted order
	 */
	public static Enumeration string( Enumeration sourceEnumeration )
		{
		SortedEnumeration stringList = 
			new SortedEnumeration( sourceEnumeration, StringComparer.getInstance() );
		return  stringList.orderedElements;
		}
	}
