/* 
 * Copyright (C) 1998 Roger Whitney <whitney@cs.sdsu.edu>
 *
 * This file is part of the San Diego State University Java Library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package sdsu.util;

/**
 * Timer measures the wall time (total lapsed-time)
 * between the call to start and stop methods. Note 
 * that wall time is not the same as cpu time consumed
 * by a process, particularly in 
 * a system that supports the threads on a multi-user system.
 * Java claims time is measuered in milliseconds.
 *
 * @version 1.0 11 January 1998 
 * @author Roger Whitney 
 *		(<a href=mailto:whitney@cs.sdsu.edu>whitney@cs.sdsu.edu</a>)
 */

public class Timer
	{
	long startTime = 0;
	long lapsedTime = 0;
	
	/**
	 * Start the timer.
	 */
	public void start()
		{
		startTime = System.currentTimeMillis();
		}
	
	/**
	 * Pause the timer.
	 */
	public void pause()
		{
		lapsedTime =  lapsedTime + 
			(System.currentTimeMillis() - startTime);
		startTime = 0;
		}

	/**
	 * Stop the timer. 
	 * Return the time measured by the timer.
	 */
	public long stop()
		{
		lapsedTime =  lapsedTime + 
			(System.currentTimeMillis() - startTime);
		startTime = 0;
		
		return time();
		}

	/**
	 * Reset the timer to zero.
	 */
	public void reset()
		{
		lapsedTime = 0;
		startTime = 0;
		}

	/**
	 * Return the time measured by the timer.
	 */
	public long time()
		{
		return lapsedTime;
		}	
	}